#include "string.h"
#include "stdio.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "samples.h"

#include "valuescript.h"
#include "conditionscript.h"
#include "actionscript.h"
#include "animscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "ActiveINTERFACE.h"
#include "ActiveMOG2Application.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

ActionSCRIPT::ActionSCRIPT(void)
{
	int i;

	command=ACTION_CMD_NONE;
	for(i=0;i<MAX_ACTION_PARAMETERS;i++) vpar[i]=0;
	cond=new ConditionSCRIPT();

	dx=0;
	dy=0;
} /* ActionSCRIPT::ActionSCRIPT */ 


ActionSCRIPT::ActionSCRIPT(ActionSCRIPT &s)
{
	int i;

	command=s.command;
	for(i=0;i<MAX_ACTION_PARAMETERS;i++) {
		vpar[i]=0;
		if (s.vpar[i]!=0) vpar[i]=new ValueSCRIPT(*s.vpar[i]);
	} /* for */ 
	cond=new ConditionSCRIPT(*s.cond);
	aparlist.Copy(s.aparlist);
	vparlist.Copy(s.vparlist);

	dx=0;
	dy=0;
} /* ActionSCRIPT::ActionSCRIPT */ 


ActionSCRIPT::~ActionSCRIPT(void)
{
	int i;
	for(i=0;i<MAX_ACTION_PARAMETERS;i++) {
		delete vpar[i];
		vpar[i]=0;
	} /* for */ 
	delete cond;
	cond=0;
} /* ActionSCRIPT::~ActionSCRIPT */ 


char *ActionSCRIPT::generate_string(void)
{
	char *str=0;

	switch(command) {
	case ACTION_CMD_NONE:
		str=new char[strlen("NONE")+1];
		sprintf(str,"NONE");
		break;
	case ACTION_CMD_SEQ:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=as->generate_string();
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("SEQ (  )")+1;

			str=new char[len];

			sprintf(str,"SEQ ( ");
			pos=strlen("SEQ ( ");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(str+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(str+pos," , ");
					pos+=3;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(str+pos," )");
		}
		break;
	case ACTION_CMD_PARALLEL:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=as->generate_string();
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("PARALLEL (  )")+1;

			str=new char[len];

			sprintf(str,"PARALLEL ( ");
			pos=strlen("PARALLEL ( ");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(str+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(str+pos," , ");
					pos+=3;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(str+pos," )");
		}		
		break;
	case ACTION_CMD_IF:
		{
			char *s,*s2,*s3;
			int len=0;

			s=cond->generate_string();
			len+=strlen(s);
		
			s2=aparlist[0]->generate_string();
			s3=aparlist[1]->generate_string();
			
			len+=strlen(s2);
			len+=strlen(s3);
			len+=strlen("IF (  )  ELSE ")+1;

			str=new char[len];

			sprintf(str,"IF ( %s ) %s ELSE %s",s,s2,s3);

			delete []s;
			delete []s2;
			delete []s3;
		}
		break;
	case ACTION_CMD_WHILE:
		{
			char *s,*s2;
			int len=0;

			s2=cond->generate_string();
			len+=strlen(s2);
		
			s=aparlist[0]->generate_string();
			
			len+=strlen(s);
			len+=strlen("WHILE (  ) {  }")+1;

			str=new char[len];

			sprintf(str,"WHILE ( %s ) { %s }",s2,s);

			delete []s;
			delete []s2;
		}
		break;
	case ACTION_CMD_WHILE_ELSE:
		{
			char *s,*s2,*s3;
			int len=0;

			s2=cond->generate_string();
			len+=strlen(s2);
		
			s=aparlist[0]->generate_string();
			len+=strlen(s);
			s3=aparlist[1]->generate_string();
			len+=strlen(s3);
			
			len+=strlen("WHILE_ELSE (  ) {  } ELSE {  }")+1;

			str=new char[len];

			sprintf(str,"WHILE_ELSE ( %s ) { %s } ELSE { %s }",s2,s,s3);

			delete []s;
			delete []s2;
			delete []s3;
		}
		break;
	case ACTION_CMD_DO:
		{
			char *s,*s2;
			int len=0;

			s2=cond->generate_string();
			len+=strlen(s2);
		
			s=aparlist[0]->generate_string();
			
			len+=strlen(s);
			len+=strlen("DO {  } WHILE (  )")+1;

			str=new char[len];

			sprintf(str,"DO { %s } WHILE ( %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;
	case ACTION_CMD_PAUSE:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("PAUSE (  )")+len+1];
			sprintf(str,"PAUSE ( %s )",s);
			delete []s;
		}
		break;
	case ACTION_CMD_ATOMIC:
		{
			char *s;
			int len=0;

	
			s=aparlist[0]->generate_string();		
			len+=strlen(s);
			len+=strlen("ATOMIC (  )")+1;

			str=new char[len];

			sprintf(str,"ATOMIC ( %s )",s);
			delete []s;
		}
		break;
	case ACTION_CMD_TERMINATE:
		{
			str=new char[strlen("TERMINATE")+1];
			sprintf(str,"TERMINATE");
		}
		break;

	case ACTION_CMD_TERMINATECHILD:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=cond->generate_string();
			len+=strlen(s2);

			str=new char[strlen("TERMINATE_CHILD (  ,  )")+len+1];
			sprintf(str,"TERMINATE_CHILD ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_GETCHILD:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=cond->generate_string();
			len+=strlen(s2);

			str=new char[strlen("GET_CHILD (  ,  )")+len+1];
			sprintf(str,"GET_CHILD ( %s , %s )",s,s2);

			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_GETPARENT:
		{
			str=new char[strlen("GET_PARENT")+1];
			sprintf(str,"GET_PARENT");
		}
		break;

	case ACTION_CMD_GETROOT:
		{
			str=new char[strlen("GET_ROOT")+1];
			sprintf(str,"GET_ROOT");
		}
		break;


	case ACTION_CMD_STATE:
		{
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("STATE (  )")+len+1];
			sprintf(str,"STATE ( %s )",s);

			delete []s;
		}
		break;

	case ACTION_CMD_SET:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("SET  = ")+len+1];
			sprintf(str,"SET %s = %s",s,s2);
			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_MOVE:
		{
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("MOVE (  ,  )")+len+1];
			sprintf(str,"MOVE ( %s , %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_CREATEOBJECT:
		{
			/* CREATEOBJECT (xoffs,yoffs,TYPE);	*/ 
			char *s,*s2,*s3,*s4;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			str=new char[strlen("CREATEOBJECT (  ,  ,  ,  )")+len+1];
			sprintf(str,"CREATEOBJECT ( %s , %s , %s , %s )",s,s2,s3,s4);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
		}
		break;
	case ACTION_CMD_CREATECHILD:
		{
			/* CREATECHILD (xoffs,yoffs,TYPE,link,part,precedence);	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			s5=vpar[4]->generate_string();
			len+=strlen(s5);

			s6=vpar[5]->generate_string();
			len+=strlen(s6);

			str=new char[strlen("CREATECHILD (  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"CREATECHILD ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	case ACTION_CMD_CREATEGLUEDCHILD:
		{
			char *s,*s2,*s3,*s4,*s5,*s6;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			s5=vpar[4]->generate_string();
			len+=strlen(s5);

			s6=vpar[5]->generate_string();
			len+=strlen(s6);

			str=new char[strlen("CREATEGLUEDCHILD (  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"CREATEGLUEDCHILD ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	case ACTION_CMD_DM_MODE_FACTOR:
		{
			/* DM_MODE_FACTOR(mode,factor);	*/ 
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("DM_MODE (  ,  )")+len+1];
			sprintf(str,"DM_MODE ( %s , %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_DM_BLINK:
		{
			/* DM_BLINK (f0,f1,speed);	*/ 
			char *s,*s2,*s3;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			str=new char[strlen("DM_BLINK (  ,  ,  )")+len+1];
			sprintf(str,"DM_BLINK ( %s , %s , %s )",s,s2,s3);
			delete []s;
			delete []s2;
            delete []s3;
		}
		break;

	case ACTION_CMD_DM_COLOR1:
		{
			/* DM_COLOR1 (r,g,b,a);	*/ 
			char *s,*s2,*s3,*s4;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			str=new char[strlen("DM_COLOR1 (  ,  ,  ,  )")+len+1];
			sprintf(str,"DM_COLOR1 ( %s , %s , %s , %s )",s,s2,s3,s4);
			delete []s;
			delete []s2;
            delete []s3;
            delete []s4;
		}
		break;


	case ACTION_CMD_DM_COLOR2:
		{
			/* DM_COLOR2 (r,g,b,a);	*/ 
			char *s,*s2,*s3,*s4;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);
                        
			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			str=new char[strlen("DM_COLOR2 (  ,  ,  ,  )")+len+1];
			sprintf(str,"DM_COLOR2 ( %s , %s , %s , %s )",s,s2,s3,s4);
			delete []s;
			delete []s2;
            delete []s3;
            delete []s4;
		}
		break;

	case ACTION_CMD_SET_INVINCIBLE:
		{
			char *s;
			int len=0;

			s=aparlist[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("SET_INVINCIBLE (  )")+len+1];
			sprintf(str,"SET_INVINCIBLE ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_LEAVELADDER:
		str=new char[strlen("LEAVELADDER")+1];
		sprintf(str,"LEAVELADDER");
		break;

	case ACTION_CMD_NEWVARIABLE:
		{
			/* NEWVARIABLE (variable_name);	*/ 
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("NEWVARIABLE (  )")+len+1];
			sprintf(str,"NEWVARIABLE ( %s )",s);
			delete []s;
		}
		break;
	case ACTION_CMD_KILLVARIABLE:
		{
			/* KILLVARIABLE (variable_name);	*/ 
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("KILLVARIABLE (  )")+len+1];
			sprintf(str,"KILLVARIABLE ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_PLAY_SAMPLE:
		{
			/* PLAY_SAMPLE (sample , volume);	*/ 
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("PLAY_SAMPLE (  ,  )")+len+1];
			sprintf(str,"PLAY_SAMPLE ( %s , %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_PLAY_SAMPLE_PAN:
		{
			/* PLAY_SAMPLE_PAN (sample , volume, x );	*/ 
			char *s,*s2,*s3;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			str=new char[strlen("PLAY_SAMPLE_PAN (  ,  ,  )")+len+1];
			sprintf(str,"PLAY_SAMPLE_PAN ( %s , %s , %s )",s,s2,s3);
			delete []s;
			delete []s2;
			delete []s3;
		}
		break;

	case ACTION_CMD_PLAY_MUSIC:
		{
			/* PLAY_MUSIC ( music_name , volume , priority );	*/ 
			char *s,*s2,*s3;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			str=new char[strlen("PLAY_MUSIC (  ,  ,  )")+len+1];
			sprintf(str,"PLAY_MUSIC ( %s , %s , %s )",s,s2,s3);
			delete []s;
			delete []s2;
			delete []s3;
		}
		break;

	case ACTION_CMD_STOP_MUSIC:
		str=new char[strlen("STOP_MUSIC")+1];
		sprintf(str,"STOP_MUSIC");
		break;

	case ACTION_CMD_GETPLAYER:
		{
			/* GETPLAYER ( n );	*/ 
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("GETPLAYER (  )")+len+1];
			sprintf(str,"GETPLAYER ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_SETPLAYERNAME:
		{
			/* SETPLAYERNAME ( n , name );	*/ 
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			s2=vpar[1]->generate_string();
			len+=strlen(s)+strlen(s2);

			str=new char[strlen("SETPLAYERNAME (  ,  )")+len+1];
			sprintf(str,"SETPLAYERNAME ( %s , %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_DRAWPIXEL:
		{
			/* DRAWPIXEL ( x,y,r,g,b,a );	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			s5=vpar[4]->generate_string();
			len+=strlen(s5);

			s6=vpar[5]->generate_string();
			len+=strlen(s6);

			str=new char[strlen("DRAWPIXEL (  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"DRAWPIXEL ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	case ACTION_CMD_DRAWLINE:
		{
			/* DRAWLINE ( x,y,r,g,b,a );	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6,*s7,*s8;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			s5=vpar[4]->generate_string();
			len+=strlen(s5);

			s6=vpar[5]->generate_string();
			len+=strlen(s6);

			s7=vpar[6]->generate_string();
			len+=strlen(s7);
			
			s8=vpar[7]->generate_string();
			len+=strlen(s8);

			str=new char[strlen("DRAWLINE (  ,  ,  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"DRAWLINE ( %s , %s , %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6,s7,s8);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
			delete []s7;
			delete []s8;
		}
		break;

	case ACTION_CMD_DRAWBOX:
		{
			/* DRAWBOX ( x,y,r,g,b,a );	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6,*s7,*s8;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			s5=vpar[4]->generate_string();
			len+=strlen(s5);

			s6=vpar[5]->generate_string();
			len+=strlen(s6);

			s7=vpar[6]->generate_string();
			len+=strlen(s7);
			
			s8=vpar[7]->generate_string();
			len+=strlen(s8);

			str=new char[strlen("DRAWBOX (  ,  ,  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"DRAWBOX ( %s , %s , %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6,s7,s8);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
			delete []s7;
			delete []s8;
		}
		break;

	case ACTION_CMD_DRAWBITMAP:
		{
			/* DRAWBITMAP ( bank,bitmap,x,y );	*/ 
			char *s,*s2,*s3,*s4;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			str=new char[strlen("DRAWBITMAP (  ,  ,  ,  )")+len+1];
			sprintf(str,"DRAWBITMAP ( %s , %s , %s , %s )",s,s2,s3,s4);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
		}
		break;

	case ACTION_CMD_CHANGEMAP:
		{
			/* CHANGEMAP ( map,x,y );	*/ 
			char *s,*s2,*s3;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			str=new char[strlen("CHANGEMAP (  ,  ,  )")+len+1];
			sprintf(str,"CHANGEMAP ( %s , %s , %s )",s,s2,s3);
			delete []s;
			delete []s2;
			delete []s3;
		}
		break;

	case ACTION_CMD_DELETE_VIEWPORT:
		{
			/* DELETE_VIEWPORT ( id );	*/ 
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("DELETE_VIEWPORT (  )")+len+1];
			sprintf(str,"DELETE_VIEWPORT ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_RESETGAME:
		str=new char[strlen("RESET_GAME")+1];
		sprintf(str,"RESET_GAME");
		break;
	case ACTION_CMD_STOPGAME:
		str=new char[strlen("STOP_GAME")+1];
		sprintf(str,"STOP_GAME");
		break;
	case ACTION_CMD_RESUMEGAME:
		str=new char[strlen("RESUME_GAME")+1];
		sprintf(str,"RESUME_GAME");
		break;

	case ACTION_CMD_SETPLAYERKEYBOARD:
		{
			/* SETPLAYERKEYBOARD ( player , up , down , left , right , f1 , f2 , f3 , f4 );	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6,*s7,*s8,*s9;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			s3=vpar[2]->generate_string();
			len+=strlen(s3);

			s4=vpar[3]->generate_string();
			len+=strlen(s4);

			s5=vpar[4]->generate_string();
			len+=strlen(s5);

			s6=vpar[5]->generate_string();
			len+=strlen(s6);

			s7=vpar[6]->generate_string();
			len+=strlen(s7);

			s8=vpar[7]->generate_string();
			len+=strlen(s8);

			s9=vpar[8]->generate_string();
			len+=strlen(s9);

			str=new char[strlen("SETPLAYERKEYBOARD (  ,  ,  ,  ,  ,  ,  ,  ,  )")+len+1];
			sprintf(str,"SETPLAYERKEYBOARD ( %s , %s , %s , %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6,s7,s8,s9);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
			delete []s7;
			delete []s8;
			delete []s9;
		}
		break;

	case ACTION_CMD_ADD_FOCUSING:
		{
			/* ADD_FOCUSING ( player ) */
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("ADD_FOCUSING (  )")+len+1];
			sprintf(str,"ADD_FOCUSING ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_SUB_FOCUSING:
		{
			/* SUB_FOCUSING ( player ) */
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("SUB_FOCUSING (  )")+len+1];
			sprintf(str,"SUB_FOCUSING ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_CLEAR_FOCUSING:
		{
			/* CLEAR_FOCUSING ( ) */
			str=new char[strlen("CLEAR_FOCUSING")+1];
			sprintf(str,"CLEAR_FOCUSING");
		}
		break;

	case ACTION_CMD_GAMEMODE:
		{
			/* GAMEMODE ( mode , file ) */
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("GAMEMODE (  ,  )")+len+1];
			sprintf(str,"GAMEMODE ( %s , %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;

	case ACTION_CMD_MENU_ADDOPTION:
		{
			/* MENU_ADDOPTION ( option ) */
			char *s;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[strlen("MENU_ADDOPTION (  )")+len+1];
			sprintf(str,"MENU_ADDOPTION ( %s )",s);
			delete []s;
		}
		break;

	case ACTION_CMD_MENU_CLEAROPTIONS:
		{
			/* MENU_CLEAROPTIONS ( ) */
			str=new char[strlen("MENU_CLEAROPTIONS")+1];
			sprintf(str,"MENU_CLEAROPTIONS");
		}
		break;

	case ACTION_CMD_MENU_SUBSTOPTION:
		{
			/* MENU_SUBSTOPTION ( option , newtext ) */
			char *s,*s2;
			int len=0;

			s=vpar[0]->generate_string();
			len+=strlen(s);
			s2=vpar[1]->generate_string();
			len+=strlen(s2);

			str=new char[strlen("MENU_SUBSTOPTION (  ,  )")+len+1];
			sprintf(str,"MENU_SUBSTOPTION ( %s , %s )",s,s2);
			delete []s;
			delete []s2;
		}
		break;


	case ACTION_CMD_CASE:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			List<char> ls;
			char *s,*s2,*s3;
			List<ValueSCRIPT> l2;
			ValueSCRIPT *vs;
			int len=0,pos;

			l.Instance(aparlist);
			l.Rewind();
			l2.Instance(vparlist);
			l2.Rewind();
			while(l.Iterate(as) &&
				  l2.Iterate(vs)) {
				s=as->generate_string();
				s2=vs->generate_string();
				s3=new char[strlen(s)+strlen(s2)+3];
				sprintf(s3,"%s %s",s2,s);
				delete []s;
				delete []s2;
				ls.Add(s3);
				len+=strlen(s3);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("CASE (  ) {  }")+1;

			s=vpar[0]->generate_string();
			len+=strlen(s);

			str=new char[len];

			sprintf(str,"CASE ( %s ) { ",s);
			pos=strlen(str);

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(str+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(str+pos," , ");
					pos+=3;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(str+pos," }");
		}
		break;

	case ACTION_CMD_RANDOM_CASE:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			List<char> ls;
			char *s,*s2,*s3;
			List<ValueSCRIPT> l2;
			ValueSCRIPT *vs;
			int len=0,pos;

			l.Instance(aparlist);
			l.Rewind();
			l2.Instance(vparlist);
			l2.Rewind();
			while(l.Iterate(as) &&
				  l2.Iterate(vs)) {
				s=as->generate_string();
				s2=vs->generate_string();
				s3=new char[strlen(s)+strlen(s2)+3];
				sprintf(s3,"%s %s",s2,s);
				delete []s;
				delete []s2;
				ls.Add(s3);
				len+=strlen(s3);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("RANDOM_CASE {  }")+1;

			str=new char[len];

			sprintf(str,"RANDOM_CASE { ",s);
			pos=strlen(str);

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(str+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(str+pos," , ");
					pos+=3;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(str+pos," }");
		}
		break;
	} /* switch */ 

	return str;
} /* ActionSCRIPT::generate_string */ 


bool ActionSCRIPT::load(FILE *fp,MOG2 *game)
{
	char tmp[80];

	if (1!=fscanf(fp,"%s",tmp)) return false;

	aparlist.Delete();

	if (strcmp(tmp,"NONE")==0) {
		command=ACTION_CMD_NONE;

	} else if (strcmp(tmp,"SEQ")==0) {
		bool end=false;
		command=ACTION_CMD_SEQ;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		while(!end) {
			ActionSCRIPT *as;
			as=new ActionSCRIPT();
			if (as->load(fp,game)) {
				aparlist.Add(as);
				if (1!=fscanf(fp,"%s",tmp)) return false;
				if (strcmp(tmp,")")==0) end=true;
			} else {
				end=true;
			} /* if */ 
		} /* while */ 

	} else if (strcmp(tmp,"PARALLEL")==0) {
		bool end=false;
		command=ACTION_CMD_PARALLEL;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		while(!end) {
			ActionSCRIPT *as;
			as=new ActionSCRIPT();
			if (as->load(fp,game)) {
				aparlist.Add(as);
				if (1!=fscanf(fp,"%s",tmp)) return false;
				if (strcmp(tmp,")")==0) end=true;
			} else {
				end=true;
			} /* if */ 
		} /* while */ 

	} else if (strcmp(tmp,"WHILE")==0) {
		command=ACTION_CMD_WHILE;
		delete cond;
		cond=new ConditionSCRIPT();
		aparlist.Add(new ActionSCRIPT());
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!cond->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"WHILE_ELSE")==0) {
		command=ACTION_CMD_WHILE_ELSE;
		delete cond;
		cond=new ConditionSCRIPT();
		aparlist.Add(new ActionSCRIPT());
		aparlist.Add(new ActionSCRIPT());
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!cond->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DO")==0) {
		command=ACTION_CMD_DO;
		delete cond;
		cond=new ConditionSCRIPT();
		aparlist.Add(new ActionSCRIPT());
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!cond->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"STATE")==0) {
		command=ACTION_CMD_STATE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"SET")==0) {
		command=ACTION_CMD_SET;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;

	} else if (strcmp(tmp,"MOVE")==0) {
		command=ACTION_CMD_MOVE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"CREATEOBJECT")==0) {
		command=ACTION_CMD_CREATEOBJECT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"CREATECHILD")==0) {
		command=ACTION_CMD_CREATECHILD;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[5];
		vpar[5]=new ValueSCRIPT();
		if (!vpar[5]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"CREATEGLUEDCHILD")==0) {
		command=ACTION_CMD_CREATEGLUEDCHILD;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[5];
		vpar[5]=new ValueSCRIPT();
		if (!vpar[5]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"PAUSE")==0) {
		command=ACTION_CMD_PAUSE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"TERMINATE")==0) {
		command=ACTION_CMD_TERMINATE;

	} else if (strcmp(tmp,"TERMINATE_CHILD")==0) {
		command=ACTION_CMD_TERMINATECHILD;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete cond;
		cond=new ConditionSCRIPT();
		aparlist.Add(new ActionSCRIPT());
		if (!cond->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"GET_CHILD")==0) {
		command=ACTION_CMD_GETCHILD;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete cond;
		cond=new ConditionSCRIPT();
		aparlist.Add(new ActionSCRIPT());
		if (!cond->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"GET_PARENT")==0) {
		command=ACTION_CMD_GETPARENT;

	} else if (strcmp(tmp,"GET_ROOT")==0) {
		command=ACTION_CMD_GETROOT;

	} else if (strcmp(tmp,"IF")==0) {
		command=ACTION_CMD_IF;
		delete cond;
		cond=new ConditionSCRIPT();
		aparlist.Add(new ActionSCRIPT());
		aparlist.Add(new ActionSCRIPT());
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!cond->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[1]->load(fp,game)) return false;

	} else if (strcmp(tmp,"ATOMIC")==0) {
		command=ACTION_CMD_ATOMIC;
		aparlist.Add(new ActionSCRIPT());
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (!aparlist[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DM_MODE")==0) {
		command=ACTION_CMD_DM_MODE_FACTOR;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DM_BLINK")==0) {
		command=ACTION_CMD_DM_BLINK;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DM_COLOR1")==0) {
		command=ACTION_CMD_DM_COLOR1;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DM_COLOR2")==0) {
		command=ACTION_CMD_DM_COLOR2;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"SET_INVINCIBLE")==0) {
		command=ACTION_CMD_SET_INVINCIBLE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		aparlist.Add(new ActionSCRIPT());
		if (!aparlist[0]->load(fp,game)) return false;		
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"LEAVELADDER")==0) {
		command=ACTION_CMD_LEAVELADDER;

	} else if (strcmp(tmp,"NEWVARIABLE")==0) {
		command=ACTION_CMD_NEWVARIABLE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"KILLVARIABLE")==0) {
		command=ACTION_CMD_KILLVARIABLE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"PLAY_SAMPLE")==0) {
		command=ACTION_CMD_PLAY_SAMPLE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"PLAY_SAMPLE_PAN")==0) {
		command=ACTION_CMD_PLAY_SAMPLE_PAN;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"PLAY_MUSIC")==0) {
		command=ACTION_CMD_PLAY_MUSIC;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"STOP_MUSIC")==0) {
		command=ACTION_CMD_STOP_MUSIC;

	} else if (strcmp(tmp,"GETPLAYER")==0) {
		command=ACTION_CMD_GETPLAYER;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"SETPLAYERNAME")==0) {
		command=ACTION_CMD_SETPLAYERNAME;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DRAWPIXEL")==0) {
		command=ACTION_CMD_DRAWPIXEL;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[5];
		vpar[5]=new ValueSCRIPT();
		if (!vpar[5]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DRAWLINE")==0) {
		command=ACTION_CMD_DRAWLINE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[5];
		vpar[5]=new ValueSCRIPT();
		if (!vpar[5]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[6];
		vpar[6]=new ValueSCRIPT();
		if (!vpar[6]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[7];
		vpar[7]=new ValueSCRIPT();
		if (!vpar[7]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DRAWBOX")==0) {
		command=ACTION_CMD_DRAWBOX;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[5];
		vpar[5]=new ValueSCRIPT();
		if (!vpar[5]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[6];
		vpar[6]=new ValueSCRIPT();
		if (!vpar[6]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[7];
		vpar[7]=new ValueSCRIPT();
		if (!vpar[7]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DRAWBITMAP")==0) {
		command=ACTION_CMD_DRAWBITMAP;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"CHANGEMAP")==0) {
		command=ACTION_CMD_CHANGEMAP;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"DELETE_VIEWPORT")==0) {
		command=ACTION_CMD_DELETE_VIEWPORT;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"RESET_GAME")==0) {
		command=ACTION_CMD_RESETGAME;
	} else if (strcmp(tmp,"STOP_GAME")==0) {
		command=ACTION_CMD_STOPGAME;
	} else if (strcmp(tmp,"RESUME_GAME")==0) {
		command=ACTION_CMD_RESUMEGAME;

	} else if (strcmp(tmp,"SETPLAYERKEYBOARD")==0) {
		command=ACTION_CMD_SETPLAYERKEYBOARD;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[2];
		vpar[2]=new ValueSCRIPT();
		if (!vpar[2]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[3];
		vpar[3]=new ValueSCRIPT();
		if (!vpar[3]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[4];
		vpar[4]=new ValueSCRIPT();
		if (!vpar[4]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[5];
		vpar[5]=new ValueSCRIPT();
		if (!vpar[5]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[6];
		vpar[6]=new ValueSCRIPT();
		if (!vpar[6]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[7];
		vpar[7]=new ValueSCRIPT();
		if (!vpar[7]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[8];
		vpar[8]=new ValueSCRIPT();
		if (!vpar[8]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"ADD_FOCUSING")==0) {
		command=ACTION_CMD_ADD_FOCUSING;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"SUB_FOCUSING")==0) {
		command=ACTION_CMD_SUB_FOCUSING;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"CLEAR_FOCUSING")==0) {
		command=ACTION_CMD_CLEAR_FOCUSING;

	} else if (strcmp(tmp,"GAMEMODE")==0) {
		command=ACTION_CMD_GAMEMODE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"CASE")==0) {
		bool end=false;
		command=ACTION_CMD_CASE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		while(!end) {
			ValueSCRIPT *vs;
			ActionSCRIPT *as;
			vs=new ValueSCRIPT();
			as=new ActionSCRIPT();
			if (vs->load(fp,game) &&
				as->load(fp,game)) {
				vparlist.Add(vs);
				aparlist.Add(as);
				if (1!=fscanf(fp,"%s",tmp)) return false;
				if (strcmp(tmp,"}")==0) end=true;
			} else {
				end=true;
			} /* if */ 
		} /* while */ 

	} else if (strcmp(tmp,"RANDOM_CASE")==0) {
		bool end=false;
		command=ACTION_CMD_RANDOM_CASE;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		while(!end) {
			ValueSCRIPT *vs;
			ActionSCRIPT *as;
			vs=new ValueSCRIPT();
			as=new ActionSCRIPT();
			if (vs->load(fp,game) &&
				as->load(fp,game)) {
				vparlist.Add(vs);
				aparlist.Add(as);
				if (1!=fscanf(fp,"%s",tmp)) return false;
				if (strcmp(tmp,"}")==0) end=true;
			} else {
				end=true;
			} /* if */ 
		} /* while */ 

	} else if (strcmp(tmp,"MENU_ADDOPTION")==0) {
		command=ACTION_CMD_MENU_ADDOPTION;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"MENU_SUBSTOPTION")==0) {
		command=ACTION_CMD_MENU_SUBSTOPTION;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[0];
		vpar[0]=new ValueSCRIPT();
		if (!vpar[0]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;
		delete vpar[1];
		vpar[1]=new ValueSCRIPT();
		if (!vpar[1]->load(fp,game)) return false;
		if (1!=fscanf(fp,"%s",tmp)) return false;

	} else if (strcmp(tmp,"MENU_CLEAROPTIONS")==0) {
		command=ACTION_CMD_MENU_CLEAROPTIONS;
	} else {

		return false;
	} /* if */ 
	return true;
} /* ActionSCRIPT::load */ 


bool ActionSCRIPT::save(FILE *fp)
{
	char *str;

	str=generate_string();

	fprintf(fp,"%s\n",str);

	delete []str;
	return true;
} /* ActionSCRIPT::save */ 




void ActionSCRIPT::init(void)
{
	timmer=0;
	aparlist.Rewind();
	first_cycle=true;
	while_else=false;

	switch(command) {
	case ACTION_CMD_SEQ:
		aparlist.Rewind();

		if (!aparlist.EmptyP()) {
			aparlist[0]->init();
		} /* if */ 
		break;
	case ACTION_CMD_PARALLEL:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			
			finished.Delete();

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				finished.Add(new bool(false));
				as->init();
			} /* while */ 
		}
		break;
	case ACTION_CMD_IF:
		cond->init();
		aparlist[0]->init();
		aparlist[1]->init();
		break;
	case ACTION_CMD_WHILE:
		cond->init();
		cond_result=true;
		first_cycle=true;
		aparlist[0]->init();
		break;
	case ACTION_CMD_WHILE_ELSE:
		cond->init();
		cond_result=true;
		first_cycle=true;
		while_else=false;
		aparlist[0]->init();
		aparlist[1]->init();
		break;
	case ACTION_CMD_DO:
		cond->init();
		cond_result=true;
		aparlist[0]->init();
		break;
	case ACTION_CMD_ATOMIC:
		aparlist[0]->init();
		break;

	case ACTION_CMD_CASE:
		first_cycle=true;
		break;

	case ACTION_CMD_RANDOM_CASE:
		first_cycle=true;
		break;

	} /* switch */ 
} /* ActionSCRIPT::init */ 


bool ActionSCRIPT::cycle(ActiveOBJECT *obj,ActiveROOM *room,ActiveMAP *map,ActiveMOG2 *game,ActiveINTERFACE *interf,ActiveMOG2Application *app,ActiveOBJECT **foreign_object,int location,KEYBOARDSTATE **LVK,int n_LVKs)
{
	timmer++;

	switch(command) {
	case ACTION_CMD_NONE:
		return true;
		break;

	case ACTION_CMD_SEQ:
		if (!aparlist.EmptyP()) {
			if (!aparlist.EndP()) {
				if (aparlist.GetObj()->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) {
					aparlist.Next();
					if (!aparlist.EndP()) aparlist.GetObj()->init();
				} /* if */ 
			} /* if */ 

			if (aparlist.EndP()) return true;
		} else {
			return true;
		} /* if */ 
		break;

	case ACTION_CMD_PARALLEL:
		{
			List<ActionSCRIPT> l;
			List<bool> l2;
			bool *bp;
			ActionSCRIPT *as;
			bool end=true;

			l.Instance(aparlist);
			l.Rewind();
			l2.Instance(finished);
			l2.Rewind();
			while(l.Iterate(as) &&
				  l2.Iterate(bp)) {
				if (*bp) {
				} else {
					if (!as->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) end=false;
																			else *bp=true;
				} /* if */ 				
			} /* while */ 

			if (end) return true;
		}
		break;

	case ACTION_CMD_IF:
		if (first_cycle) {
			cond_result=cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
			first_cycle=false;
		} /* if */ 
		if (cond_result) {
			if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) return true;
		} else {
			if (aparlist[1]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) return true;
		} /* if */ 
		break;
	case ACTION_CMD_WHILE:
		if (first_cycle) {
			first_cycle=false;
			cond_result=cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
			if (cond_result) {
				if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) {
					aparlist[0]->init();
					first_cycle=true;
				} /* if */ 
			} else {
				return true;
			} /* if */ 
		} else {
			cond_result=cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
			if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) {
				aparlist[0]->init();
				first_cycle=true;
			} /* if */ 
		} /* if */ 
		break;
	case ACTION_CMD_WHILE_ELSE:
		if (!while_else) {
			if (first_cycle) {
				first_cycle=false;
				cond_result=cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
				if (cond_result) {
					if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) {
						aparlist[0]->init();
						first_cycle=true;
					} /* if */ 
				} else {
					while_else=true;
					return aparlist[1]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);;
				} /* if */ 
			} else {
				cond_result=cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
				if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) {
					aparlist[0]->init();
					first_cycle=true;
				} /* if */ 
			} /* if */ 
		} else {
			return aparlist[1]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
		} /* if */ 
		break;
	case ACTION_CMD_DO:
		if (aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)) {
			cond_result=cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
			if (cond_result) aparlist[0]->init();
		} else {
			cond->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
		} /* if */ 
		if (!cond_result) return true;
		break;
	case ACTION_CMD_PAUSE:
		{
			int itmp;
			if (vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				timmer>=itmp) {
				return true;
			} /* if */ 

			return false;
		}
		break;

	case ACTION_CMD_ATOMIC:
		while(!aparlist[0]->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs));
		return true;
		break;

	case ACTION_CMD_TERMINATE:
		if (obj!=0) obj->terminate=true;
		if (obj==0 && app!=0) app->terminated=true;
		return true;
		break;
	case ACTION_CMD_TERMINATECHILD:
		{
			List<ActiveOBJECT> l;
			ActiveOBJECT *o;
			char *stmp;

			if (obj!=0 &&
				vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
			
				l.Instance(obj->glued_children);
				l.Rewind();
				while(l.Iterate(o)) {
					if (strcmp(o->object->name,stmp)==0) {
						*foreign_object=o;
						if (cond->cycle(obj,room,map,game,interf,app,foreign_object,location,0,0)) {
							o->terminate=true;
						} /* if */ 
						*foreign_object=0;
					} /* if */ 
				} /* while */ 

				l.Instance(obj->independent_children);
				l.Rewind();
				while(l.Iterate(o)) {
					if (strcmp(o->object->name,stmp)==0) {
						*foreign_object=o;
						if (cond->cycle(obj,room,map,game,interf,app,foreign_object,location,0,0)) {
							o->terminate=true;
						} /* if */ 
						*foreign_object=0;
					} /* if */ 
				} /* while */ 

			} /* if */ 

		}
		return true;
		break;

	case ACTION_CMD_GETCHILD:
		{
			List<ActiveOBJECT> l;
			ActiveOBJECT *o;
			char *stmp;

			if (obj!=0 &&
				vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
			
				*foreign_object=0;
				l.Instance(obj->glued_children);
				l.Rewind();
				while(l.Iterate(o)) {
					if (strcmp(o->object->name,stmp)==0) {
						*foreign_object=o;
						if (cond->cycle(obj,room,map,game,interf,app,foreign_object,location,0,0)) {
							return true;
						} /* if */ 
						*foreign_object=0;
					} /* if */ 
				} /* while */ 

				*foreign_object=0;
				l.Instance(obj->independent_children);
				l.Rewind();
				while(l.Iterate(o)) {
					if (strcmp(o->object->name,stmp)==0) {
						*foreign_object=o;
						if (cond->cycle(obj,room,map,game,interf,app,foreign_object,location,0,0)) {
							return true;
						} /* if */ 
						*foreign_object=0;
					} /* if */ 
				} /* while */ 

			} /* if */ 
		}
		return true;
		break;

	case ACTION_CMD_GETPARENT:
		{
			*foreign_object=0;
			if (obj!=0) {		
				*foreign_object=obj->parent;
			} /* if */ 
		}
		return true;
		break;

	case ACTION_CMD_GETROOT:
		{
			*foreign_object=obj;
			if (obj!=0) {
				while((*foreign_object)->parent!=0) *foreign_object=(*foreign_object)->parent;
			} /* if */ 
		}
		return true;
		break;


	case ACTION_CMD_STATE:
		{
			int itmp;

			if (obj!=0 &&
				vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				itmp>=0 && 
				((obj->object->control!=CTRL_CHARACTER && itmp<obj->object->n_states) ||
				 (obj->object->control==CTRL_CHARACTER && itmp<CPOS_NSTATES))) {
				obj->executed_state_change=true;
				if (obj->object->control==CTRL_CHARACTER) {
					if (itmp==CPOS_JUMPLEFT) {
						obj->jumping_inertia=-obj->walking_speed;
						obj->state_timmer=0;
					} /* if */ 
					if (itmp==CPOS_JUMPRIGHT) {
						obj->jumping_inertia=obj->walking_speed;
						obj->state_timmer=0;
					} /* if */ 
				} else {
					obj->g_state_action[itmp]->init();
				} /* if */ 
				if (obj->highest_priority_special()!=-1) {
					obj->g_state_finished=false;
					obj->state_before_special=itmp;					
				} else {
					int j;
					obj->state=itmp;
					obj->state_timmer=0;
					obj->state_change=true;
					obj->g_state_finished=false;
					if (obj->object->control==CTRL_CHARACTER) {
						for(j=0;j<obj->object->nparts;j++) obj->change_tile(obj->state_anim[j][obj->state]->get_tile(),j);
					} else {
						for(j=0;j<obj->object->nparts;j++) obj->change_tile(obj->g_state_anim[j][obj->state]->get_tile(),j);
					} /* if */ 
				} /* if */ 
			} /* if */ 
		}
		return true;
		break;

	case ACTION_CMD_SET:
		{
			int type;
			int itmp;
			char *stmp;
			ActiveOBJECT *otmp,*oloc=0;
			int interface_num=-1;
			int vloc=VALUE_VAR_NONE;
			char *vname;


			if (vpar[0]->command==VALUE_CMD_VARIABLE ||
				vpar[0]->command==VALUE_CMD_MAKE_VARIABLE ||
				vpar[0]->command==VALUE_CMD_DOT) {
				if (vpar[0]->command==VALUE_CMD_VARIABLE) {
					vloc=vpar[0]->vlocation;
					vname=vpar[0]->vname;
				} else if (vpar[0]->command==VALUE_CMD_MAKE_VARIABLE) {
					char *stmp3;
					vloc=VALUE_VAR_NONE;
					if (vpar[0]->vpar[0]->cycle(0,&vname,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
						vpar[0]->vpar[1]->cycle(0,&stmp3,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
						if (strcmp(stmp3,"object")==0) vloc=VALUE_VAR_OBJECT;
						if (strcmp(stmp3,"room")==0) vloc=VALUE_VAR_ROOM;
						if (strcmp(stmp3,"map")==0) vloc=VALUE_VAR_MAP;
						if (strcmp(stmp3,"game")==0) vloc=VALUE_VAR_GAME;
						if (strcmp(stmp3,"foreign")==0) vloc=VALUE_VAR_FOREIGN;
						if (strcmp(stmp3,"interface")==0) vloc=VALUE_VAR_INTERFACE;
						if (strcmp(stmp3,"application")==0) vloc=VALUE_VAR_APPLICATION;
						if (stmp3[0]=='i' &&
							stmp3[1]=='n' &&
							stmp3[2]=='t' &&
							stmp3[3]=='e' &&
							stmp3[4]=='r' &&
							stmp3[5]=='f' &&
							stmp3[6]=='a' &&
							stmp3[7]=='c' &&
							stmp3[8]=='e' &&
							stmp3[9]==' ') {
							int i;

							if (1==sscanf(stmp3+10,"%i",&i)) {
								if (i>=0 && i<game->viewports.Length()) {
									vloc=VALUE_VAR_INTERFACE;
									interface_num=i;
								} /* if */ 
							} /* if */ 
						} /* if */ 
					} /* if */ 
				} else {
					/* DOT */ 
					if (vpar[0]->vpar[0]->cycle(0,0,&oloc,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_OBJECT &&
						vpar[0]->vpar[1]->cycle(0,&vname,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
						vloc=VALUE_VAR_DOT;
					} /* if */ 
					
				} /* if */ 

				type=vpar[1]->cycle(&itmp,&stmp,&otmp,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
				if (type==VALUE_INTEGER) {
					switch(vloc) {
					case VALUE_VAR_OBJECT:		
						if (obj!=0) obj->set_variable_value(vname,itmp);
						break;
					case VALUE_VAR_ROOM:
						if (room!=0) room->set_variable_value(vname,itmp);
						break;
					case VALUE_VAR_MAP:
						if (map!=0) map->set_variable_value(vname,itmp);
						break;
					case VALUE_VAR_GAME:
						if (game!=0) game->set_variable_value(vname,itmp);
						break;
					case VALUE_VAR_INTERFACE:
						if (interface_num==-1) {
							if (interf!=0) interf->set_variable_value(vname,itmp);
						} else {
							if (game!=0) game->viewports[interface_num]->current_interface->set_variable_value(vname,itmp);
						} /* if */ 
						break;
					case VALUE_VAR_APPLICATION:
						if (app!=0) app->set_variable_value(vname,itmp);
						break;
					case VALUE_VAR_FOREIGN:
						if (*foreign_object!=0) {
							List<ActiveOBJECT> oil;
							ActiveOBJECT *oi;

							(*foreign_object)->set_variable_value(vname,itmp);
							oil.Instance((*foreign_object)->other_instances);
							oil.Rewind();
							while(oil.Iterate(oi)) {
								oi->set_variable_value(vname,itmp);
							} /* while */ 
						} /* if */ 
						break;
					case VALUE_VAR_DOT:
						if (oloc!=0) {
							oloc->set_variable_value(vname,itmp);
						} /* if */ 
						break;
					} /* switch */ 					
				} /* if */ 

				if (type==VALUE_STRING) {
					switch(vloc) {
					case VALUE_VAR_OBJECT:	
						if (obj!=0) obj->set_variable_value(vname,stmp);
						break;
					case VALUE_VAR_ROOM:
						if (room!=0) room->set_variable_value(vname,stmp);
						break;
					case VALUE_VAR_MAP:
						if (map!=0) map->set_variable_value(vname,stmp);
						break;
					case VALUE_VAR_GAME:
						if (game!=0) game->set_variable_value(vname,stmp);
						break;
					case VALUE_VAR_INTERFACE:
						if (interface_num==-1) {
							if (interf!=0) interf->set_variable_value(vname,stmp);
						} else {
							if (game!=0) game->viewports[interface_num]->current_interface->set_variable_value(vname,stmp);
						} /* if */ 

						break;
					case VALUE_VAR_APPLICATION:
						if (app!=0) app->set_variable_value(vname,stmp);
						break;
					case VALUE_VAR_FOREIGN:
						if (*foreign_object!=0) {
							List<ActiveOBJECT> oil;
							ActiveOBJECT *oi;

							(*foreign_object)->set_variable_value(vname,stmp);
							oil.Instance((*foreign_object)->other_instances);
							oil.Rewind();
							while(oil.Iterate(oi)) {
								oi->set_variable_value(vname,stmp);
							} /* while */ 
						} /* if */ 

						break;
					case VALUE_VAR_DOT:
						if (oloc!=0) {
							oloc->set_variable_value(vname,stmp);
						} /* if */ 
						break;
					} /* switch */ 					
				} /* if */ 

				if (type==VALUE_OBJECT) {
					switch(vloc) {
					case VALUE_VAR_OBJECT:		
						if (obj!=0) obj->set_variable_value(vname,otmp);
						break;
					case VALUE_VAR_ROOM:
						if (room!=0) room->set_variable_value(vname,otmp);
						break;
					case VALUE_VAR_MAP:
						if (map!=0) map->set_variable_value(vname,otmp);
						break;
					case VALUE_VAR_GAME:
						if (game!=0) game->set_variable_value(vname,otmp);
						break;
					case VALUE_VAR_INTERFACE:
						if (interface_num==-1) {
							if (interf!=0) interf->set_variable_value(vname,otmp);
						} else {
							if (game!=0) game->viewports[interface_num]->current_interface->set_variable_value(vname,otmp);
						} /* if */ 
						break;
					case VALUE_VAR_APPLICATION:
						if (app!=0) app->set_variable_value(vname,otmp);
						break;
					case VALUE_VAR_FOREIGN:
						if (*foreign_object!=0) {
							List<ActiveOBJECT> oil;
							ActiveOBJECT *oi;

							(*foreign_object)->set_variable_value(vname,otmp);
							oil.Instance((*foreign_object)->other_instances);
							oil.Rewind();
							while(oil.Iterate(oi)) {
								oi->set_variable_value(vname,otmp);
							} /* while */ 
						} /* if */ 
						break;
					case VALUE_VAR_DOT:
						if (oloc!=0) {
							oloc->set_variable_value(vname,otmp);
						} /* if */ 
						break;
					} /* switch */ 					
				} /* if */ 
			} /* if */ 
		}
		return true;
		break;

	case ACTION_CMD_MOVE:
		{
			int itmp,itmp2;

			if (obj!=0 &&
				vpar[0]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				obj->precision_move(itmp,itmp2);
			} /* if */ 
		}
		return true;
		break;

	case ACTION_CMD_CREATEOBJECT:
		{
			int itmp,itmp2,itmp3;
			char *stmp;

			if ((obj!=0 || room!=0 || interf!=0) &&
				vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&itmp2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&itmp3,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				/* Search object: */ 
				OBJECT *o=game->find_object(stmp);

				/* Create the object: */ 
				if (o!=0) {					
					ActiveOBJECT *ao;
					int x=itmp;
					int y=itmp2;

					if (interf==0 && room!=0 && obj!=0) {
						x+=(obj->x*100);
						y+=(obj->y*100);
					} /* if */ 

					x=x/100;
					y=y/100;

					
					if (itmp3<0) itmp3=0;
					if (itmp3>=MAX_OBJECT_PLANES) itmp3=MAX_OBJECT_PLANES-1;

					if (interf!=0) {
						ao=new ActiveOBJECT(x,y,-1,o,0,interf);
						interf->objects_toadd[itmp3].Add(ao);
					} else {
						ao=new ActiveOBJECT(x,y,-1,o,room,0);
						room->objects_toadd[itmp3].Add(ao);
					} /* if */ 
                                        
					if (obj!=0) obj->objects_created.Add(ao);
					*foreign_object=ao;
				} /* if */ 

			} /* if */ 
			return true;
		}
		break;
	case ACTION_CMD_CREATECHILD:
		{
			int x_offs,y_offs,part,link,prec;
			char *stmp;

			if (obj!=0 &&
				vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&x_offs,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&y_offs,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&part,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&link,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[5]->cycle(&prec,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				/* Search object: */ 
				OBJECT *o=game->find_object(stmp);

				/* Create the child object: */ 
				if (o!=0) {
					int dx=0,dy=0,lx=0,ly=0;
					ActiveOBJECT *ao;

					if (obj->object->ntiles>0) {
						obj->object->get_part_position(part,obj->tile,&dx,&dy);
						obj->object->get_link(obj->tile,part,link,&lx,&ly);
					} /* if */ 

					int x=((obj->x+dx+lx)*100)+x_offs;
					int y=((obj->y+dy+ly)*100)+y_offs;

					x=x/100;
					y=y/100;

					ao=new ActiveOBJECT(x,y,-1,o,obj->room,obj->interf);
					ao->relative_precedence=prec;
					ao->parent=obj;
					
					obj->independent_children.Add(ao);
					obj->objects_created.Add(ao);
					*foreign_object=ao;
				} /* if */ 

			} /* if */ 
			return true;
		}
		break;
	case ACTION_CMD_CREATEGLUEDCHILD:
		{
			int x_offs,y_offs,part,link,prec;
			char *stmp;

			if (obj!=0 &&
				vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&x_offs,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&y_offs,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&part,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&link,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[5]->cycle(&prec,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				/* Search object: */ 
				OBJECT *o=game->find_object(stmp);

				/* Create the child object: */ 
				if (o!=0) {
					int dx=0,dy=0,lx=0,ly=0;
					ActiveOBJECT *ao;

					if (obj->object->ntiles>0) {
						obj->object->get_part_position(part,obj->tile,&dx,&dy);
						obj->object->get_link(obj->tile,part,link,&lx,&ly);
					} /* if */ 

					int x=((obj->x+dx+lx)*100)+x_offs;
					int y=((obj->y+dy+ly)*100)+y_offs;

					x=x/100;
					y=y/100;

					ao=new ActiveOBJECT(x,y,-1,o,obj->room,obj->interf);
					ao->relative_precedence=prec;
					ao->parent=obj;
					ao->glued_part=part;
					ao->glued_link=link;
					
					obj->glued_children.Add(ao);
					obj->objects_created.Add(ao);
					*foreign_object=ao;
					ao->recompute_glued_position();
				} /* if */ 

			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DM_MODE_FACTOR:
		{
			int itmp;
			char *stmp;

			if (obj!=0 &&
				vpar[0]->cycle(0,&stmp,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&itmp,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				obj->dm_drawing_mode=0;
				if (strcmp(stmp,"shaded")==0) obj->dm_drawing_mode=1;
				if (strcmp(stmp,"bicolor")==0) obj->dm_drawing_mode=2;
				if (strcmp(stmp,"reflect")==0) obj->dm_drawing_mode=3;
				obj->dm_factor=itmp;
				if (obj->dm_factor<0) obj->dm_factor=0;
				if (obj->dm_factor>100) obj->dm_factor=100;
				obj->dm_blink_speed=0;
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DM_BLINK:
		{
			int f0,f1,speed;

			if (obj!=0 &&
				vpar[0]->cycle(&f0,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&f1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&speed,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				/* ... */ 
				if (f0<0) f0=0;
				if (f0>100) f0=100;
				if (f1<0) f1=0;
				if (f1>100) f1=100;
				if (speed<-100) speed=-100;
				if (speed>100) speed=100;
				obj->dm_blink_f0=f0;
				obj->dm_blink_f1=f1;
				obj->dm_blink_speed=speed;
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DM_COLOR1:
		{
			int r,g,b,a;

			if (obj!=0 &&
				vpar[0]->cycle(&r,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&g,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&b,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&a,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				obj->dm_color1_r=r;
				if (obj->dm_color1_r<-1) obj->dm_color1_r=0;
				if (obj->dm_color1_r>255) obj->dm_color1_r=255;
				obj->dm_color1_g=g;
				if (obj->dm_color1_g<-1) obj->dm_color1_g=0;
				if (obj->dm_color1_g>255) obj->dm_color1_g=255;
				obj->dm_color1_b=b;
				if (obj->dm_color1_b<-1) obj->dm_color1_b=0;
				if (obj->dm_color1_b>255) obj->dm_color1_b=255;
				obj->dm_color1_a=a;
				if (obj->dm_color1_a<-1) obj->dm_color1_a=0;
				if (obj->dm_color1_a>255) obj->dm_color1_a=255;
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DM_COLOR2:
		{
			int r,g,b,a;

			if (obj!=0 &&
				vpar[0]->cycle(&r,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&g,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&b,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&a,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				obj->dm_color2_r=r;
				if (obj->dm_color2_r<-1) obj->dm_color2_r=-1;
				if (obj->dm_color2_r>255) obj->dm_color2_r=255;
				obj->dm_color2_g=g;
				if (obj->dm_color2_g<-1) obj->dm_color2_g=-1;
				if (obj->dm_color2_g>255) obj->dm_color2_g=255;
				obj->dm_color2_b=b;
				if (obj->dm_color2_b<-1) obj->dm_color2_b=-1;
				if (obj->dm_color2_b>255) obj->dm_color2_b=255;
				obj->dm_color2_a=a;
				if (obj->dm_color2_a<-1) obj->dm_color2_a=-1;
				if (obj->dm_color2_a>255) obj->dm_color2_a=255;
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_SET_INVINCIBLE:
		if (obj!=0) {
			if (obj->invincible_script==0) obj->invincible_script=new ActionSCRIPT();
			obj->invincible_script->copy_state(aparlist[0]);
			obj->invincible_script->init();
			return true;
		}
		break;

	case ACTION_CMD_LEAVELADDER:
		if (obj!=0) obj->in_ladder=0;
		return true;
		break;

	case ACTION_CMD_NEWVARIABLE:
		{
			char *var_name;

			if (vpar[0]->cycle(0,&var_name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
				VARIABLE *v;

				v=new VARIABLE(var_name,0);
				if (obj!=0) {
					obj->object_variables.Add(v);
				} else {
					if (room!=0) {
						room->room_variables.Add(v);
					} else {
						if (map!=0) {
							map->map_variables.Add(v);
						} else {
							if (game!=0) {
								game->game_variables.Add(v);
							} /* if */ 
						} /* if */ 
					} /* if */ 
				} /* if */ 
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_KILLVARIABLE:
		{
			char *var_name;

			if (vpar[0]->cycle(0,&var_name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
				VARIABLE *v,*v2;
				List<VARIABLE> l;


				if (obj!=0) {
					l.Instance(obj->object_variables);
					l.Rewind();
					v2=0;
					while(v2==0 && l.Iterate(v)) {
						if (v->cmp(var_name)) v2=v;
					} /* while */ 

					obj->object_variables.DeleteElement(v2);
					delete v2;
				} else {
					if (room!=0) {
						l.Instance(room->room_variables);
						l.Rewind();
						v2=0;
						while(v2==0 && l.Iterate(v)) {
							if (v->cmp(var_name)) v2=v;
						} /* while */ 

						room->room_variables.DeleteElement(v2);
						delete v2;
					} else {
						if (map!=0) {
							l.Instance(map->map_variables);
							l.Rewind();
							v2=0;
							while(v2==0 && l.Iterate(v)) {
								if (v->cmp(var_name)) v2=v;
							} /* while */ 

							map->map_variables.DeleteElement(v2);
							delete v2;							
						} else {
							if (game!=0) {
								l.Instance(game->game_variables);
								l.Rewind();
								v2=0;
								while(v2==0 && l.Iterate(v)) {
									if (v->cmp(var_name)) v2=v;
								} /* while */ 

								game->game_variables.DeleteElement(v2);
								delete v2;
							} /* if */ 
						} /* if */ 
					} /* if */ 
				} /* if */ 
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_PLAY_SAMPLE:
		{
			SAMPLE *s=0;
			char *sample_name;
			int volume;

			if (vpar[0]->get_command()==VALUE_CMD_SAMPLE) {
				s=vpar[0]->get_sample();
			} else {
				if (vpar[0]->cycle(0,&sample_name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
					SAMPLE *s2;
					List<SAMPLE> l;

					l.Instance(game->active_game->samples);
					l.Rewind();
					s=0;
					while(s==0 && l.Iterate(s2)) {
						if (s2->cmp(sample_name)) s=s2;
					} /* while */ 
				} /* if */ 
			} /* if */ 

			if (s!=0 &&
				app!=0 &&
				vpar[1]->cycle(&volume,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				PLAYSAMPLE *ps;

				if (volume<0) volume=0;
				if (volume>100) volume=100;

				ps=new PLAYSAMPLE();
				ps->sample=s;
				ps->volume=(volume*127)/100;
				ps->pan=0;
				ps->room=room;
				ps->placed=false;
				if (obj!=0 && room!=0) {
					ps->placed=true;
					ps->x=obj->x;
					ps->y=obj->y;
				} /* if */ 
				app->play_samples.Add(ps);
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_PLAY_SAMPLE_PAN:
		{
			SAMPLE *s=0;
			char *sample_name;
			int volume,x;

			if (vpar[0]->get_command()==VALUE_CMD_SAMPLE) {
				s=vpar[0]->get_sample();
			} else {
				if (vpar[0]->cycle(0,&sample_name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
					SAMPLE *s2;
					List<SAMPLE> l;

					l.Instance(game->active_game->samples);
					l.Rewind();
					s=0;
					while(s==0 && l.Iterate(s2)) {
						if (s2->cmp(sample_name)) s=s2;
					} /* while */ 
				} /* if */ 
			} /* if */ 

			if (s!=0 &&
				app!=0 &&				
				vpar[1]->cycle(&volume,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&x,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				PLAYSAMPLE *ps;

				if (volume<0) volume=0;
				if (volume>100) volume=100;
				if (x<0) x=0;
				if (x>room->room->get_dx()*TILESIZE) x=room->room->get_dx()*TILESIZE;

				ps=new PLAYSAMPLE();
				ps->sample=s;
				ps->volume=(volume*127)/100;
				if (room!=0) ps->pan=int( (long(x)*200L) / long(room->room->get_dx()*TILESIZE) ) - 100;
				        else ps->pan=0;
				ps->room=room;
				ps->placed=false;
				if (obj!=0 && room!=0) {
					ps->placed=true;
					ps->x=obj->x;
					ps->y=obj->y;
				} /* if */ 
				app->play_samples.Add(ps);
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_PLAY_MUSIC:
		{
			char *command;
			int volume,priority;

			if (app!=0 &&
				vpar[0]->cycle(0,&command,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&volume,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&priority,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				char tmp[256];
				char tmp2[256];
				char *part_name;
				int i,j,k;

				if (priority>=app->music_priority) {

					app->music_priority=0;
					delete []app->music_playing;
					app->music_playing=0;
					app->music_parts.Delete();
					delete []app->music_sequence;
					app->music_sequence=0;
					app->music_sequence_ip=0;
					app->music_part=0;
					init_music(app);

					k=0;
					tmp2[k++]='S';

					strcpy(tmp,"");
					j=strlen(tmp);

					for(i=0;command[i]!=0;i++) {
						if (command[i]=='#') {
							bool part_added=false;

							/* a command! */ 
							if (j>0) {
								/* end of a part: */ 
								tmp[j]=0;
								part_name=new char[strlen(tmp)+1];
								strcpy(part_name,tmp);
								app->music_parts.Add(part_name);
								strcpy(tmp,"");
								j=strlen(tmp);
								part_added=true;
							} /* if */ 

							/* Read command: */ 
							i++;
							if (command[i]=='E') {								
								tmp2[k++]='E';
							} /* if */ 
							if (command[i]=='G') {
								i++;
								if (command[i]>='0' && command[i]<='9') {
									tmp2[k++]='G';
									tmp2[k++]=command[i];
								} /* if */ 							
							} /* if */ 
							if (command[i]=='S') {								
								if (part_added) tmp2[k++]='S';
							} /* if */ 
						} else {
							tmp[j++]=command[i];
						} /* if */ 
					} /* for */ 

					if (j>0) {
						/* end of a part: */ 
						tmp[j]=0;
						part_name=new char[strlen(tmp)+1];
						strcpy(part_name,tmp);
						app->music_parts.Add(part_name);
					} /* if */ 

					app->music_priority=priority;
					app->music_volume=volume;

					tmp2[k]=0;
					app->music_sequence=new char[strlen(tmp2)+1];
					strcpy(app->music_sequence,tmp2);
					app->music_sequence_ip=0;
				} /* if */ 

			} /* if */ 
			return true;
		}
				
		break;

	case ACTION_CMD_STOP_MUSIC:
		
		if (app!=0) {				
			app->music_priority=0;
			delete []app->music_playing;
			app->music_playing=0;
			app->music_parts.Delete();
			delete []app->music_sequence;
			app->music_sequence=0;
			app->music_sequence_ip=0;
			app->music_part=0;
		} /* if */ 
		return true;

		break;

	case ACTION_CMD_GETPLAYER:
		{
			int i1;
			if (vpar[0]->cycle(&i1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				if (game!=0 &&
					game->player_objects.Length()>i1) {
					*foreign_object=game->player_objects[i1];
				} /* if */ 
			} /* if */ 
			return true;		
		}
		break;

	case ACTION_CMD_SETPLAYERNAME:
		{
			int i1;
			char *name;

			if (vpar[0]->cycle(&i1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(0,&name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
				if (game!=0 &&
					game->player_objects.Length()>i1) {
					char *s;

					s=game->player_names[i1];
					delete s;
					s=new char[strlen(name)+1];
					strcpy(s,name);
					game->player_names.GoTo(i1);
					game->player_names.SetObj(s);
					game->player_names.Rewind();
				} /* if */ 
			} /* if */ 
			return true;
			
		}
		break;

	case ACTION_CMD_DRAWPIXEL:
		{
			int x1,y1,r,g,b,a;
			if ((interf!=0 || room!=0) &&
				vpar[0]->cycle(&x1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&y1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&r,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&g,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&b,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[5]->cycle(&a,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				DRAWCOMMAND *dc;

				dc=new DRAWCOMMAND();
				dc->command=0;
				dc->x1=x1;
				dc->y1=y1;
				dc->x2=0;
				dc->y2=0;
				dc->r=r;
				dc->g=g;
				dc->b=b;
				dc->a=a;

				if (interf!=0) {
					interf->draw_commands.Add(dc);
				} else {
					room->draw_commands.Add(dc);
				} /* if */ 
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DRAWLINE:
		{
			int x1,y1,x2,y2,r,g,b,a;
			if ((interf!=0 || room!=0) &&
				vpar[0]->cycle(&x1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&y1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&x2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&y2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&r,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[5]->cycle(&g,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[6]->cycle(&b,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[7]->cycle(&a,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				DRAWCOMMAND *dc;

				dc=new DRAWCOMMAND();
				dc->command=1;
				dc->x1=x1;
				dc->y1=y1;
				dc->x2=x2;
				dc->y2=y2;
				dc->r=r;
				dc->g=g;
				dc->b=b;
				dc->a=a;

				if (interf!=0) {
					interf->draw_commands.Add(dc);
				} else {
					room->draw_commands.Add(dc);
				} /* if */ 
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DRAWBOX:
		{
			int x1,y1,x2,y2,r,g,b,a;
			if ((interf!=0 || room!=0) &&
				vpar[0]->cycle(&x1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&y1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&x2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&y2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&r,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[5]->cycle(&g,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[6]->cycle(&b,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[7]->cycle(&a,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				DRAWCOMMAND *dc;

				dc=new DRAWCOMMAND();
				dc->command=2;
				dc->x1=x1;
				dc->y1=y1;
				dc->x2=x2;
				dc->y2=y2;
				dc->r=r;
				dc->g=g;
				dc->b=b;
				dc->a=a;

				if (interf!=0) {
					interf->draw_commands.Add(dc);
				} else {
					room->draw_commands.Add(dc);
				} /* if */ 
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_DRAWBITMAP:
		{
			int x1,y1,bank,bitmap;
			if ((interf!=0 || room!=0) &&
				vpar[0]->cycle(&bank,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&bitmap,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&x1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&y1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				DRAWCOMMAND *dc;

				dc=new DRAWCOMMAND();
				dc->command=3;
				dc->x1=x1;
				dc->y1=y1;
				dc->x2=bank;
				dc->y2=bitmap;

				if (interf!=0) {
					interf->draw_commands.Add(dc);
				} else {
					room->draw_commands.Add(dc);
				} /* if */ 
			} /* if */ 
			return true;
		}
		break;

	case ACTION_CMD_CHANGEMAP:
		{
			char *map_name;
			int x,y;
			if (game!=0 &&
				vpar[0]->cycle(0,&map_name,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING &&
				vpar[1]->cycle(&x,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&y,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				ActiveOBJECT *target_o=0;
				ROOM *target_room=0;
				MAP *target_map=0;

				if (obj!=0) {
					/* Executed from an ActiveOBJECT: */ 
					target_o=obj;
				} else {
					if (room!=0 && (*foreign_object)!=0) {
						/* Executed from an ActiveROOM to change the foreign_object: */ 
						target_o=*foreign_object;
					} /* if */ 
				} /* if */ 

				if (target_o!=0) {
					/* look for the destination map: */ 
					List<ActiveMAP> l;
					ActiveMAP *am;
					List<MAP> l2;
					MAP *m;
					bool found=false;

					l.Instance(game->active_maps);
					l.Rewind();
					while(!found && l.Iterate(am)) {
						if (am->map->cmp(map_name)) {
							target_map=am->map;
							found=true;
						} /* if */ 
					} /* while */ 

					if (!found) {
						l2.Instance(game->active_game->maps);
						l2.Rewind();
						while(!found && l2.Iterate(m)) {
							if (m->cmp(map_name)) {
								target_map=m;
								found=true;
							} /* if */ 
						} /* while */ 
					} /* if */ 

					if (target_map!=0) {
						/* Look for the destination room: */ 
						target_room=target_map->get_room(x/TILESIZE,y/TILESIZE);
						
						if (target_room!=0) {
							ActiveOBJECT *oi;
							ActiveROOM *r;
							int p;

							r=game->add_room(target_room);

							target_o->other_instances.Rewind();
							while(!target_o->other_instances.EmptyP()) {
								oi=target_o->other_instances.ExtractIni();
								oi->room->remove_object_references(oi);
								oi->get_map()->remove_object_references(oi);
								delete oi;
							} /* while */ 

							p=target_o->get_object_plane();

							target_o->room->remove_object_references(target_o);
							target_o->get_map()->remove_object_references(target_o);
							target_o->room=r;

							r->objects[p].Add(target_o);
							r->nonpassive_objects[p].Add(target_o);
							target_o->x=target_o->old_x=x-(target_room->x*TILESIZE);
							target_o->y=target_o->old_y=y-(target_room->y*TILESIZE);
						} /* if */ 
					} /* if */ 

				} /* if */ 
			} /* if */ 
			return true;
		} 
		break;

	case ACTION_CMD_DELETE_VIEWPORT:
		{
			int id;

			if (vpar[0]->cycle(&id,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				VIEWPORT *v=game->viewports[id];
				game->viewports.DeletePosition(id);
				delete v;
			} /* if */ 

			return true;
		}
		break;

	case ACTION_CMD_RESETGAME:
		if (game!=0) game->reset();
		return true;
		break;

	case ACTION_CMD_STOPGAME:
		if (game!=0) game->paused=true;
		return true;
		break;

	case ACTION_CMD_RESUMEGAME:
		if (game!=0) game->paused=false;
		return true;
		break;

	case ACTION_CMD_SETPLAYERKEYBOARD:
		{
			int id,up,down,left,right,f1,f2,f3,f4;
			if (vpar[0]->cycle(&id,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[1]->cycle(&up,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[2]->cycle(&down,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[3]->cycle(&left,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[4]->cycle(&right,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[5]->cycle(&f1,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[6]->cycle(&f2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[7]->cycle(&f3,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
				vpar[8]->cycle(&f4,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				if (game!=0 &&
					id>=0 &&
					game->player_objects.Length()>id) {
					ActiveOBJECT *p;

					p=game->player_objects[id];
					if (p!=0) {
						p->up_key=up;
						p->down_key=down;
						p->left_key=left;
						p->right_key=right;
						p->fire1_key=f1;
						p->fire2_key=f2;
						p->fire3_key=f3;
						p->fire4_key=f4;
					} /* if */ 
				} /* if */ 
			} /* if */ 
		}
		return true;
		break;

	case ACTION_CMD_ADD_FOCUSING:
		{
			int retval;
			int i;
			ActiveOBJECT *o;

			if (obj->object->control==CTRL_GAMEWINDOW) {
				retval=vpar[0]->cycle(&i,0,&o,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);

				if (retval==VALUE_INTEGER) {
					if (!obj->gw_focusing.MemberRefP(game->player_objects[i])) 
						obj->gw_focusing.Add(game->player_objects[i]);
				} else {
					if (retval==VALUE_OBJECT) {
						if (!obj->gw_focusing.MemberRefP(o))
							obj->gw_focusing.Add(o);
					} /* if */ 
				} /* if */ 
			} /* if */ 

			return true;
		}
		break;
	
	case ACTION_CMD_SUB_FOCUSING:
		{
			int retval;
			int i;
			ActiveOBJECT *o;

			if (obj->object->control==CTRL_GAMEWINDOW) {
				retval=vpar[0]->cycle(&i,0,&o,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);

				if (retval==VALUE_INTEGER) {
					obj->gw_focusing.DeleteElement(game->player_objects[i]);
				} else {
					if (retval==VALUE_OBJECT) {
						obj->gw_focusing.DeleteElement(o);
					} /* if */ 
				} /* if */ 
			} /* if */ 

			return true;
		}
		break;

	case ACTION_CMD_CLEAR_FOCUSING:
		if (obj->object->control==CTRL_GAMEWINDOW) {
			while(!obj->gw_focusing.EmptyP()) obj->gw_focusing.ExtractIni();
		} /* if */ 

		return true;
		break;

	case ACTION_CMD_CASE:
		if (first_cycle) {
			int i;
			int v,v2;
			bool found=false;

			first_cycle=false;
			if (vpar[0]->cycle(&v,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
				List<ValueSCRIPT> ls;
				ValueSCRIPT *vs;

				ls.Instance(vparlist);
				ls.Rewind();
				i=0;
				while(!found && ls.Iterate(vs)) {
					if (vs->cycle(&v2,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
						if (v==v2) {
							aparlist.GoTo(i);
							aparlist.GetObj()->init();
							found=true;
						} /* if */ 
					} /* if */ 
					i++;
				} /* while */ 

			} /* if */ 

			if (!found) return true;
		} /* if */ 

		return aparlist.GetObj()->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
		break;

	case ACTION_CMD_RANDOM_CASE:
		if (first_cycle) {
			int i,v;
			int accum=0;
			bool found=false;
			first_cycle=false;
			List<ValueSCRIPT> ls;
			ValueSCRIPT *vs;
			int *vals;

			if (vparlist.Length()>0) {

				vals=new int[vparlist.Length()];
				for(i=0;i<vparlist.Length();i++) vals[i]=0;

				ls.Instance(vparlist);
				ls.Rewind();
				i=0;
				while(ls.Iterate(vs)) {
					if (vs->cycle(&v,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER) {
						accum+=v;
						vals[i]=v;
					} /* if */ 
					i++;
				} /* while */ 

				if (accum>0) {
					v=app->random(0,accum-1);

					accum=0;
					for(i=0;!found && i<vparlist.Length();i++) {
						if (v<accum+vals[i]) {
							aparlist.GoTo(i);
							aparlist.GetObj()->init();
							found=true;
						} /* if */ 
						accum+=vals[i];
					} /* for */ 
				} else {
					return true;
				} /* if */ 
			} /* if */ 

			if (!found) return true;
		} /* if */ 

		return aparlist.GetObj()->cycle(obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs);
		break;

	case ACTION_CMD_MENU_ADDOPTION:
		{
			char *s,*s2;

			if (obj->object->control==CTRL_MENU) {
				if (vpar[0]->cycle(0,&s,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
					s2=new char[strlen(s)+1];
					strcpy(s2,s);
					obj->menu_items.Add(s2);
				} /* if */ 
			} /* if */ 

			return true;
		}
		break;

	case ACTION_CMD_MENU_CLEAROPTIONS:
		if (obj->object->control==CTRL_MENU) {
			obj->menu_items.Delete();
		} /* if */ 

		return true;
		break;

	case ACTION_CMD_MENU_SUBSTOPTION:
		{
			int i;
			char *s,*s2;

			if (obj->object->control==CTRL_MENU) {
				if (vpar[0]->cycle(&i,0,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_INTEGER &&
					vpar[1]->cycle(0,&s,0,obj,room,map,game,interf,app,foreign_object,location,LVK,n_LVKs)==VALUE_STRING) {
					s2=new char[strlen(s)+1];
					strcpy(s2,s);
					obj->menu_items.GoTo(i);
					delete obj->menu_items.GetObj();
					obj->menu_items.SetObj(s2);
				} /* if */ 
			} /* if */ 

			return true;
		}
		break;


	case ACTION_CMD_GAMEMODE:
		return true;
		break;


	} /* switch */ 

	return false;
} /* ActionSCRIPT::cycle */ 


void ActionSCRIPT::copy_state(ActionSCRIPT *s)
{
	int i;
	timmer=s->timmer;
	first_cycle=s->first_cycle;
	while_else=s->while_else;

	command=s->command;

	for(i=0;i<MAX_ACTION_PARAMETERS;i++) {
		if (vpar[i]==0) {
			if (s->vpar[i]!=0) {
				vpar[i]=new ValueSCRIPT();
				vpar[i]->copy_state(s->vpar[i]);
			} /* if */ 
		} else {
			if (s->vpar[i]==0) {
				delete vpar[i];
				vpar[i]=0;
			} else {
				vpar[i]->copy_state(s->vpar[i]);
			} /* if */ 
		} /* if */ 
	} /* for */ 

	cond->copy_state(s->cond);
	cond_result=s->cond_result;

	if (aparlist.Length()!=s->aparlist.Length()) {
		aparlist.Delete();
		aparlist.Copy(s->aparlist);
	} else {
		List<ActionSCRIPT> l,l2;
		ActionSCRIPT *ap,*ap2;

		l.Instance(aparlist);
		l2.Instance(s->aparlist);
		l.Rewind();
		l2.Rewind();

		while(l.Iterate(ap) &&
			  l2.Iterate(ap2)) {
			ap->copy_state(ap2);
		} /* while */ 
	} /* if */ 
	aparlist.Synchronize(&(s->aparlist));

	if (vparlist.Length()!=s->vparlist.Length()) {
		vparlist.Delete();
		vparlist.Copy(s->vparlist);
	} else {
		List<ValueSCRIPT> l,l2;
		ValueSCRIPT *vp,*vp2;

		l.Instance(vparlist);
		l2.Instance(s->vparlist);
		l.Rewind();
		l2.Rewind();

		while(l.Iterate(vp) &&
			  l2.Iterate(vp2)) {
			vp->copy_state(vp2);
		} /* while */ 
	} /* if */ 
	vparlist.Synchronize(&(s->vparlist));
	
	dx=0;
	dy=0;
} /* ActionSCRIPT::copy_state */ 


void ActionSCRIPT::simplify(void)
{
	ActionSCRIPT *tmp,*tmp2;
	List<ActionSCRIPT> l,l2,ltmp,todelete;

	l.Instance(aparlist);
	l.Rewind();
	while(l.Iterate(tmp)) {
		if (tmp->command==ACTION_CMD_DELETE) {
			if (command==ACTION_CMD_SEQ ||
				command==ACTION_CMD_PARALLEL) {
				todelete.Add(tmp);
			} else {
				tmp->command=ACTION_CMD_NONE;
			} /* if */ 
		} else {
			tmp->simplify();
		} /* if */ 
	} /* while */ 

	while(!todelete.EmptyP()) {
		tmp=todelete.ExtractIni();
		aparlist.DeleteElement(tmp);
		delete tmp;
	} /* while */ 


	switch(command) {
	case ACTION_CMD_DELETE:
		command=ACTION_CMD_NONE;
		break;

	case ACTION_CMD_SEQ:
	case ACTION_CMD_PARALLEL:
		{			
			ltmp.Delete();

			l.Instance(aparlist);
			l.Rewind();
			while(l.Iterate(tmp)) {
				if (tmp->command==command) {
					l2.Instance(tmp->aparlist);
					l2.Rewind();
					while(l2.Iterate(tmp2)) {
						ltmp.Add(new ActionSCRIPT(*tmp2));
					} /* while */ 
				} else {
					ltmp.Add(new ActionSCRIPT(*tmp));
				} /* if */ 
			} /* while */ 

			aparlist.Delete();
			aparlist.Copy(ltmp);
			ltmp.Delete();

			if (aparlist.Length()==1) {
				tmp=aparlist.ExtractIni();

				copy_state(tmp);
				delete tmp;
			} /* if */ 
		}
		break;
	case ACTION_CMD_IF:
	case ACTION_CMD_DO:
	case ACTION_CMD_WHILE:
	case ACTION_CMD_WHILE_ELSE:
		cond->simplify();
		break;
	default:
		break;
	} /* switch */ 

} /* ActionSCRIPT::simplify */ 



bool ActionSCRIPT::uses_sample(SAMPLE *s)
{
	int i;
	ActionSCRIPT *tmp;
	List<ActionSCRIPT> l;

	l.Instance(aparlist);
	l.Rewind();
	while(l.Iterate(tmp)) 
		if (tmp->uses_sample(s)) return true;

	for(i=0;i<MAX_CONDITION_PARAMETERS;i++) if (vpar[i]!=0) if (vpar[i]->uses_sample(s)) return true;

	switch(command) {
	case ACTION_CMD_IF:
	case ACTION_CMD_DO:
	case ACTION_CMD_WHILE:
	case ACTION_CMD_WHILE_ELSE:
		if (cond->uses_sample(s)) return true;
		break;
	default:
		break;
	} /* switch */ 

	return false;
} /* uses_sample */ 

